/*=========================================================================
| promana Interface Library
|--------------------------------------------------------------------------
| Copyright (c) 2002-2015 Total Phase, Inc.
| All rights reserved.
| www.totalphase.com
|
| Redistribution and use in source and binary forms, with or without
| modification, are permitted provided that the following conditions
| are met:
|
| - Redistributions of source code must retain the above copyright
|   notice, this list of conditions and the following disclaimer.
|
| - Redistributions in binary form must reproduce the above copyright
|   notice, this list of conditions and the following disclaimer in the
|   documentation and/or other materials provided with the distribution.
|
| - Neither the name of Total Phase, Inc. nor the names of its
|   contributors may be used to endorse or promote products derived from
|   this software without specific prior written permission.
|
| THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
| "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
| LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
| FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
| COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
| INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
| BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
| LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
| CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
| LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
| ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
| POSSIBILITY OF SUCH DAMAGE.
|--------------------------------------------------------------------------
| To access promana through the API:
|
| 1) Use one of the following shared objects:
|      promana.so      --  Linux/Mac OSX shared object
|      promana.dll     --  Windows dynamic link library
|
| 2) Along with one of the following language modules:
|      promana.c/h     --  C/C++ API header file and interface module
|      promana_py.py   --  Python API
|      promana.cs      --  C# .NET source
 ========================================================================*/

using System;
using System.Reflection;
using System.Runtime.InteropServices;

//[assembly: AssemblyTitleAttribute("Promana .NET binding")]
//[assembly: AssemblyDescriptionAttribute(".NET binding for Promana")]
//[assembly: AssemblyCompanyAttribute("Total Phase, Inc.")]
//[assembly: AssemblyProductAttribute("Promana")]
//[assembly: AssemblyCopyrightAttribute("Total Phase, Inc. 2016")]

namespace TotalPhase {

public enum PromiraAppStatus : int {
    /* General codes (0 to -99) */
    PA_APP_OK                       =   0,
    PA_APP_UNABLE_TO_LOAD_LIBRARY   =  -1,
    PA_APP_UNABLE_TO_LOAD_DRIVER    =  -2,
    PA_APP_UNABLE_TO_LOAD_FUNCTION  =  -3,
    PA_APP_INCOMPATIBLE_LIBRARY     =  -4,
    PA_APP_INCOMPATIBLE_DEVICE      =  -5,
    PA_APP_COMMUNICATION_ERROR      =  -6,
    PA_APP_UNABLE_TO_OPEN           =  -7,
    PA_APP_UNABLE_TO_CLOSE          =  -8,
    PA_APP_INVALID_HANDLE           =  -9,
    PA_APP_CONFIG_ERROR             = -10,
    PA_APP_MEMORY_ALLOC_ERROR       = -11,
    PA_APP_UNABLE_TO_INIT_SUBSYSTEM = -12,
    PA_APP_INVALID_LICENSE          = -13,
    PA_APP_UNKNOWN_PROTOCOL         = -14,
    PA_APP_STILL_ACTIVE             = -15,
    PA_APP_INACTIVE                 = -16,
    PA_APP_FUNCTION_NOT_AVAILABLE   = -17,
    PA_APP_READ_EMPTY               = -18,

    PA_APP_TIMEOUT                  = -31,
    PA_APP_CONNECTION_LOST          = -32,

    PA_APP_QUEUE_FULL               = -50,

    PA_APP_UNKNOWN_CMD              = -83
}

public enum PromiraProtocol : int {
    PA_PROTOCOL_NONE = 0,
    PA_PROTOCOL_ESPI = 1
}

public enum PromiraTriggerMode : int {
    PA_TRIGGER_MODE_EVENT     = 0,
    PA_TRIGGER_MODE_IMMEDIATE = 1
}

public enum PromiraCaptureStatus : int {
    PA_CAPTURE_STATUS_UNKNOWN          = -1,
    PA_CAPTURE_STATUS_INACTIVE         =  0,
    PA_CAPTURE_STATUS_PRE_TRIGGER      =  1,
    PA_CAPTURE_STATUS_PRE_TRIGGER_SYNC =  2,
    PA_CAPTURE_STATUS_POST_TRIGGER     =  3,
    PA_CAPTURE_STATUS_TRANSFER         =  4,
    PA_CAPTURE_STATUS_COMPLETE         =  5
}

public enum PromiraMatchType : int {
    PA_MATCH_TYPE_DISABLED  = 0,
    PA_MATCH_TYPE_EQUAL     = 1,
    PA_MATCH_TYPE_NOT_EQUAL = 2
}

public enum PromiraSpiIOMode : int {
    PA_SPI_IO_UNKNOWN  = -1,
    PA_SPI_IO_STANDARD =  0,
    PA_SPI_IO_DUAL     =  2,
    PA_SPI_IO_QUAD     =  4
}

public enum PromiraEspiAlertPin : int {
    PA_ESPI_ALERT_UNKNOWN = 0,
    PA_ESPI_ALERT_PIN     = 1,
    PA_ESPI_ALERT_IO1     = 2
}

public enum PromiraEspiAlign : int {
    PA_ESPI_ALIGN_UNKNOWN    = 0,
    PA_ESPI_ALIGN_64_BYTES   = 1,
    PA_ESPI_ALIGN_128_BYTES  = 2,
    PA_ESPI_ALIGN_256_BYTES  = 3,
    PA_ESPI_ALIGN_512_BYTES  = 4,
    PA_ESPI_ALIGN_1024_BYTES = 5,
    PA_ESPI_ALIGN_2048_BYTES = 6,
    PA_ESPI_ALIGN_4096_BYTES = 7
}

public enum PromiraEspiChannel : int {
    PA_ESPI_CHANNEL_UNKNOWN = -1,
    PA_ESPI_CHANNEL_PERIF   =  0,
    PA_ESPI_CHANNEL_VW      =  1,
    PA_ESPI_CHANNEL_OOB     =  2,
    PA_ESPI_CHANNEL_FLASH   =  3,
    PA_ESPI_CHANNEL_INDEP   =  4
}

public enum PromiraEspiEnumFreq : int {
    PA_ESPI_ENUM_FREQ_UNKNOWN = -1,
    PA_ESPI_ENUM_FREQ_20MHZ   =  0,
    PA_ESPI_ENUM_FREQ_25MHZ   =  1,
    PA_ESPI_ENUM_FREQ_33MHZ   =  2,
    PA_ESPI_ENUM_FREQ_50MHZ   =  3,
    PA_ESPI_ENUM_FREQ_66MHZ   =  4
}


public class PromanaApi {

/*=========================================================================
| HELPER FUNCTIONS / CLASSES
 ========================================================================*/
static long tp_min(long x, long y) { return x < y ? x : y; }

private class GCContext {
    GCHandle[] handles;
    int index;
    public GCContext () {
        handles = new GCHandle[16];
        index   = 0;
    }
    public void add (GCHandle gch) {
        handles[index] = gch;
        index++;
    }
    public void free () {
        while (index != 0) {
            index--;
            handles[index].Free();
        }
    }
}

/*=========================================================================
| VERSION
 ========================================================================*/
[DllImport ("promana")]
private static extern int pa_app_c_version ();

public const int PA_APP_API_VERSION    = 0x010a;   // v1.10
public const int PA_APP_REQ_SW_VERSION = 0x010a;   // v1.10

private static short PA_APP_SW_VERSION;
private static short PA_APP_REQ_API_VERSION;
private static bool  PA_APP_LIBRARY_LOADED;

static PromanaApi () {
    PA_APP_SW_VERSION      = (short)(pa_app_c_version() & 0xffff);
    PA_APP_REQ_API_VERSION = (short)((pa_app_c_version() >> 16) & 0xffff);
    PA_APP_LIBRARY_LOADED  = 
        ((PA_APP_SW_VERSION >= PA_APP_REQ_SW_VERSION) &&
         (PA_APP_API_VERSION >= PA_APP_REQ_API_VERSION));
}

/*=========================================================================
| STATUS CODES
 ========================================================================*/
/*
 * All API functions return an integer which is the result of the
 * transaction, or a status code if negative.  The status codes are
 * defined as follows:
 */
// enum PromiraAppStatus  (from declaration above)
//     PA_APP_OK                       =   0
//     PA_APP_UNABLE_TO_LOAD_LIBRARY   =  -1
//     PA_APP_UNABLE_TO_LOAD_DRIVER    =  -2
//     PA_APP_UNABLE_TO_LOAD_FUNCTION  =  -3
//     PA_APP_INCOMPATIBLE_LIBRARY     =  -4
//     PA_APP_INCOMPATIBLE_DEVICE      =  -5
//     PA_APP_COMMUNICATION_ERROR      =  -6
//     PA_APP_UNABLE_TO_OPEN           =  -7
//     PA_APP_UNABLE_TO_CLOSE          =  -8
//     PA_APP_INVALID_HANDLE           =  -9
//     PA_APP_CONFIG_ERROR             = -10
//     PA_APP_MEMORY_ALLOC_ERROR       = -11
//     PA_APP_UNABLE_TO_INIT_SUBSYSTEM = -12
//     PA_APP_INVALID_LICENSE          = -13
//     PA_APP_UNKNOWN_PROTOCOL         = -14
//     PA_APP_STILL_ACTIVE             = -15
//     PA_APP_INACTIVE                 = -16
//     PA_APP_FUNCTION_NOT_AVAILABLE   = -17
//     PA_APP_READ_EMPTY               = -18
//     PA_APP_TIMEOUT                  = -31
//     PA_APP_CONNECTION_LOST          = -32
//     PA_APP_QUEUE_FULL               = -50
//     PA_APP_UNKNOWN_CMD              = -83


/*=========================================================================
| GENERAL TYPE DEFINITIONS
 ========================================================================*/
/* Connection handle type definition */
/* typedef PromiraConnectionHandle => int */

/*
 * Version matrix.
 *
 * This matrix describes the various version dependencies
 * of Promira components.  It can be used to determine
 * which component caused an incompatibility error.
 *
 * All version numbers are of the format:
 *   (major << 8) | minor
 *
 * ex. v1.20 would be encoded as:  0x0114
 */
[StructLayout(LayoutKind.Sequential)]
public struct PromiraAppVersion {
    /* Software, firmware, and hardware versions. */
    public ushort software;
    public ushort firmware;
    public ushort hardware;

    /* Firmware requires that software must be >= this version. */
    public ushort sw_req_by_fw;

    /* Software requires that firmware must be >= this version. */
    public ushort fw_req_by_sw;

    /* Software requires that the API interface must be >= this version. */
    public ushort api_req_by_sw;
}


/*=========================================================================
| GENERAL API
 ==========================================================================
| Connect to the application
|
| Returns a connection handle, which is guaranteed to be
| greater than zero if it is vali*/
public static int pa_app_connect (
    string  net_addr
)
{
    if (!PA_APP_LIBRARY_LOADED) return (int)PromiraAppStatus.PA_APP_INCOMPATIBLE_LIBRARY;
    return net_pa_app_connect(net_addr);
}

/* Disconnect from the application */
public static int pa_app_disconnect (
    int  conn
)
{
    if (!PA_APP_LIBRARY_LOADED) return (int)PromiraAppStatus.PA_APP_INCOMPATIBLE_LIBRARY;
    return net_pa_app_disconnect(conn);
}

/*
 * Return the version matrix for the device attached to the
 * given handle.  If the handle is 0 or invalid, only the
 * software and required api versions are set.
 */
public static int pa_app_version (
    int                    conn,
    ref PromiraAppVersion  version
)
{
    if (!PA_APP_LIBRARY_LOADED) return (int)PromiraAppStatus.PA_APP_INCOMPATIBLE_LIBRARY;
    return net_pa_app_version(conn, ref version);
}

/*
 * Return the status string for the given status code.
 * If the code is not valid or the library function cannot
 * be loaded, return a NULL string.
 */
public static string pa_app_status_string (
    int  status
)
{
    if (!PA_APP_LIBRARY_LOADED) return null;
    return Marshal.PtrToStringAnsi(net_pa_app_status_string(status));
}

/*
 * Return the device features as a bit-mask of values, or
 * an error code if the handle is not valid.
 */
public const int PA_FEATURE_NONE = 0x00000000;
public const int PA_FEATURE_ESPI = 0x00000001;
public static int pa_app_features (
    int  conn
)
{
    if (!PA_APP_LIBRARY_LOADED) return (int)PromiraAppStatus.PA_APP_INCOMPATIBLE_LIBRARY;
    return net_pa_app_features(conn);
}


/*=========================================================================
| POWER API
 ==========================================================================
| Configure the target power pin*/
public const byte PA_PHY_TARGET_POWER_NONE = 0x00;
public const byte PA_PHY_TARGET_POWER_TARGET1_5V = 0x01;
public const byte PA_PHY_TARGET_POWER_TARGET1_3V = 0x05;
public const byte PA_PHY_TARGET_POWER_TARGET2 = 0x02;
public const byte PA_PHY_TARGET_POWER_BOTH = 0x03;
public const byte PA_PHY_TARGET_POWER_QUERY = 0x80;
public static int pa_phy_target_power (
    int   conn,
    byte  config
)
{
    if (!PA_APP_LIBRARY_LOADED) return (int)PromiraAppStatus.PA_APP_INCOMPATIBLE_LIBRARY;
    return net_pa_phy_target_power(conn, config);
}

/* Configure the power of output signal. */
public const float PA_PHY_LEVEL_SHIFT_QUERY = -1;
public static float pa_phy_level_shift (
    int    conn,
    float  level
)
{
    if (!PA_APP_LIBRARY_LOADED) return 0;
    return net_pa_phy_level_shift(conn, level);
}


/*=========================================================================
| CAPTURE API
 ==========================================================================
| Protocol cod*/
// enum PromiraProtocol  (from declaration above)
//     PA_PROTOCOL_NONE = 0
//     PA_PROTOCOL_ESPI = 1

/* Trigger modes */
// enum PromiraTriggerMode  (from declaration above)
//     PA_TRIGGER_MODE_EVENT     = 0
//     PA_TRIGGER_MODE_IMMEDIATE = 1

/* Start capturing */
public static int pa_capture_start (
    int                 conn,
    PromiraProtocol     protocol,
    PromiraTriggerMode  trig_mode
)
{
    if (!PA_APP_LIBRARY_LOADED) return (int)PromiraAppStatus.PA_APP_INCOMPATIBLE_LIBRARY;
    return net_pa_capture_start(conn, protocol, trig_mode);
}

/* Stop capturing data */
public static int pa_capture_stop (
    int  conn
)
{
    if (!PA_APP_LIBRARY_LOADED) return (int)PromiraAppStatus.PA_APP_INCOMPATIBLE_LIBRARY;
    return net_pa_capture_stop(conn);
}

/* Trigger capture manually */
public static int pa_capture_trigger (
    int  conn
)
{
    if (!PA_APP_LIBRARY_LOADED) return (int)PromiraAppStatus.PA_APP_INCOMPATIBLE_LIBRARY;
    return net_pa_capture_trigger(conn);
}

/*
 * Capture status; general across protocols but used in
 * protocol-specific capture-status-query functions as well
 */
// enum PromiraCaptureStatus  (from declaration above)
//     PA_CAPTURE_STATUS_UNKNOWN          = -1
//     PA_CAPTURE_STATUS_INACTIVE         =  0
//     PA_CAPTURE_STATUS_PRE_TRIGGER      =  1
//     PA_CAPTURE_STATUS_PRE_TRIGGER_SYNC =  2
//     PA_CAPTURE_STATUS_POST_TRIGGER     =  3
//     PA_CAPTURE_STATUS_TRANSFER         =  4
//     PA_CAPTURE_STATUS_COMPLETE         =  5

/* Wait for capture to trigger */
public static int pa_capture_trigger_wait (
    int                       conn,
    int                       timeout_ms,
    ref PromiraCaptureStatus  status
)
{
    if (!PA_APP_LIBRARY_LOADED) return (int)PromiraAppStatus.PA_APP_INCOMPATIBLE_LIBRARY;
    return net_pa_capture_trigger_wait(conn, timeout_ms, ref status);
}

[StructLayout(LayoutKind.Sequential)]
public struct PromiraCaptureBufferStatus {
    public uint pretrig_remaining_kb;
    public uint pretrig_total_kb;
    public uint capture_remaining_kb;
    public uint capture_total_kb;
}

public static int pa_capture_status (
    int                             conn,
    ref PromiraCaptureStatus        status,
    ref PromiraCaptureBufferStatus  buf_status
)
{
    if (!PA_APP_LIBRARY_LOADED) return (int)PromiraAppStatus.PA_APP_INCOMPATIBLE_LIBRARY;
    return net_pa_capture_status(conn, ref status, ref buf_status);
}

/* General constants */
public static int pa_capture_buffer_config (
    int   conn,
    uint  pretrig_kb,
    uint  capture_kb
)
{
    if (!PA_APP_LIBRARY_LOADED) return (int)PromiraAppStatus.PA_APP_INCOMPATIBLE_LIBRARY;
    return net_pa_capture_buffer_config(conn, pretrig_kb, capture_kb);
}

public static int pa_capture_buffer_config_query (
    int       conn,
    ref uint  pretrig_kb,
    ref uint  capture_kb
)
{
    if (!PA_APP_LIBRARY_LOADED) return (int)PromiraAppStatus.PA_APP_INCOMPATIBLE_LIBRARY;
    return net_pa_capture_buffer_config_query(conn, ref pretrig_kb, ref capture_kb);
}


/*=========================================================================
| DIGITAL INPUT/OUTPUT CONFIGURATION
 ========================================================================*/
public const uint PA_DIGITAL_DIR_INPUT = 0;
public const uint PA_DIGITAL_DIR_OUTPUT = 1;
public const uint PA_DIGITAL_ACTIVE_LOW = 0;
public const uint PA_DIGITAL_ACTIVE_HIGH = 1;
/* Configure the digital I/Os */
public static int pa_digital_io_config (
    int   conn,
    uint  enable,
    uint  direction,
    uint  polarity
)
{
    if (!PA_APP_LIBRARY_LOADED) return (int)PromiraAppStatus.PA_APP_INCOMPATIBLE_LIBRARY;
    return net_pa_digital_io_config(conn, enable, direction, polarity);
}

public static int pa_digital_io_config_query (
    int       conn,
    ref uint  enable,
    ref uint  direction,
    ref uint  polarity
)
{
    if (!PA_APP_LIBRARY_LOADED) return (int)PromiraAppStatus.PA_APP_INCOMPATIBLE_LIBRARY;
    return net_pa_digital_io_config_query(conn, ref enable, ref direction, ref polarity);
}


/*=========================================================================
| COMMON CAPTURE TRIGGER OPTIONS
 ==========================================================================
| Packet matching mod*/
// enum PromiraMatchType  (from declaration above)
//     PA_MATCH_TYPE_DISABLED  = 0
//     PA_MATCH_TYPE_EQUAL     = 1
//     PA_MATCH_TYPE_NOT_EQUAL = 2


/*=========================================================================
| COMMON STATUS/EVENT FOR READ
 ========================================================================*/
[StructLayout(LayoutKind.Sequential)]
public struct PromiraReadInfo {
    public ulong timestamp;
    public ulong duration;
    public uint  status;
    public uint  events;
}

/* Status */
public const uint PA_READ_OK = 0;
public const uint PA_READ_ERR_CODE_MASK = 0x000000ff;
/* Event */
public const uint PA_EVENT_DIGITAL_INPUT_MASK = 0x00000fff;
public const uint PA_EVENT_DIGITAL_INPUT = 0x00001000;
public const uint PA_EVENT_SLAVE_ID_MASK = 0x30000000;
public const uint PA_EVENT_SLAVE0 = 0x00000000;
public const uint PA_EVENT_SLAVE1 = 0x10000000;
public const uint PA_EVENT_SLAVE2 = 0x20000000;
public const uint PA_EVENT_SLAVE3 = 0x30000000;
public const uint PA_EVENT_TRIGGER = 0x80000000;
/* SPI IO Mode for SPI and eSPI */
// enum PromiraSpiIOMode  (from declaration above)
//     PA_SPI_IO_UNKNOWN  = -1
//     PA_SPI_IO_STANDARD =  0
//     PA_SPI_IO_DUAL     =  2
//     PA_SPI_IO_QUAD     =  4


/*=========================================================================
| eSPI API
 ========================================================================*/
// enum PromiraEspiAlertPin  (from declaration above)
//     PA_ESPI_ALERT_UNKNOWN = 0
//     PA_ESPI_ALERT_PIN     = 1
//     PA_ESPI_ALERT_IO1     = 2

// enum PromiraEspiAlign  (from declaration above)
//     PA_ESPI_ALIGN_UNKNOWN    = 0
//     PA_ESPI_ALIGN_64_BYTES   = 1
//     PA_ESPI_ALIGN_128_BYTES  = 2
//     PA_ESPI_ALIGN_256_BYTES  = 3
//     PA_ESPI_ALIGN_512_BYTES  = 4
//     PA_ESPI_ALIGN_1024_BYTES = 5
//     PA_ESPI_ALIGN_2048_BYTES = 6
//     PA_ESPI_ALIGN_4096_BYTES = 7

[StructLayout(LayoutKind.Sequential)]
public struct PromiraEspiOperatingCfg {
    public PromiraSpiIOMode    io_mode;
    public PromiraEspiAlertPin alert_pin;
    public PromiraEspiAlign    perif_max_req_size;
    public PromiraEspiAlign    perif_max_payload;
    public byte                vw_max_count;
    public PromiraEspiAlign    oob_max_payload;
    public PromiraEspiAlign    flash_max_req_size;
    public PromiraEspiAlign    flash_max_payload;
}

/* Set the operating configuration of eSPI system */
public static int pa_espi_operating_config (
    int                      conn,
    byte                     slave_id,
    PromiraEspiOperatingCfg  cfg
)
{
    if (!PA_APP_LIBRARY_LOADED) return (int)PromiraAppStatus.PA_APP_INCOMPATIBLE_LIBRARY;
    return net_pa_espi_operating_config(conn, slave_id, ref cfg);
}

/* Read the operation configuration that eSPI system is using */
public static int pa_espi_operating_config_read (
    int                          conn,
    byte                         slave_id,
    ref PromiraEspiOperatingCfg  cfg
)
{
    if (!PA_APP_LIBRARY_LOADED) return (int)PromiraAppStatus.PA_APP_INCOMPATIBLE_LIBRARY;
    return net_pa_espi_operating_config_read(conn, slave_id, ref cfg);
}

/* Status - error */
public const uint PA_READ_ESPI_ERR_TYPE_MASK = 0x000000c0;
public const uint PA_READ_ESPI_ERR_TYPE_MASTER = 0x00000080;
public const uint PA_READ_ESPI_ERR_TYPE_SLAVE = 0x00000040;
public const uint PA_READ_ESPI_ERR_TYPE_MISC = 0x000000c0;
public const uint PA_READ_ESPI_ERR_FATAL_MASK = 0x00000020;
/* Master Error Code - Fatal */
public const uint PA_READ_ESPI_MST_INVALID_RSP_CODE = 0x000000a0;
public const uint PA_READ_ESPI_MST_INVALID_CYCLE_TYPE = 0x000000a1;
public const uint PA_READ_ESPI_MST_NO_RSP = 0x000000a3;
public const uint PA_READ_ESPI_MST_RSP_FATAL = 0x000000a4;
public const uint PA_READ_ESPI_MST_PERIF_PAYLOAD = 0x000000a5;
public const uint PA_READ_ESPI_MST_PERIF_REQ_SIZE = 0x000000a6;
public const uint PA_READ_ESPI_MST_PERIF_4K_XING = 0x000000a7;
public const uint PA_READ_ESPI_MST_VW_MAX_COUNT = 0x000000a8;
public const uint PA_READ_ESPI_MST_OOB_PAYLOAD = 0x000000a9;
public const uint PA_READ_ESPI_MST_FLASH_PAYLOAD = 0x000000aa;
public const uint PA_READ_ESPI_MST_FLASH_REQ_SIZE = 0x000000ab;
public const uint PA_READ_ESPI_MST_PERIF_PAYLOAD_SIZE = 0x000000ad;
public const uint PA_READ_ESPI_MST_FLASH_PAYLOAD_SIZE = 0x000000ae;
/* Master Error Code - Non Fatal */
public const uint PA_READ_ESPI_MST_RSP_NON_FATAL = 0x00000080;
/* Slave Error Code - Fatal */
public const uint PA_READ_ESPI_SLV_PUT_WO_FREE = 0x00000060;
public const uint PA_READ_ESPI_SLV_GET_WHEN_UNAVAIL = 0x00000061;
public const uint PA_READ_ESPI_SLV_PERIF_PAYLOAD = 0x0000006a;
public const uint PA_READ_ESPI_SLV_PERIF_REQ_SIZE = 0x00000062;
public const uint PA_READ_ESPI_SLV_PERIF_4K_XING = 0x00000063;
public const uint PA_READ_ESPI_SLV_VW_MAX_COUNT = 0x00000064;
public const uint PA_READ_ESPI_SLV_OOB_PAYLOAD = 0x00000065;
public const uint PA_READ_ESPI_SLV_FLASH_PAYLOAD = 0x00000066;
public const uint PA_READ_ESPI_SLV_FLASH_REQ_SIZE = 0x00000067;
public const uint PA_READ_ESPI_SLV_PERIF_PAYLOAD_SIZE = 0x00000068;
public const uint PA_READ_ESPI_SLV_FLASH_PAYLOAD_SIZE = 0x00000069;
/* Slave Error Code - Non Fatal */
public const uint PA_READ_ESPI_SLV_INVALID_CMD = 0x00000040;
public const uint PA_READ_ESPI_SLV_INVALID_CYCLE_TYPE = 0x00000041;
/* Miscellaneous Error Code */
public const uint PA_READ_ESPI_PARTIAL_BYTE = 0x000000e0;
public const uint PA_READ_ESPI_RESET_WHILE_CS = 0x000000e1;
public const uint PA_READ_ESPI_ALERT_WHILE_CS = 0x000000e2;
public const uint PA_READ_ESPI_INVALID_LENGTH = 0x000000e3;
public const uint PA_READ_ESPI_MORE_THAN_ONE_CS = 0x000000e4;
public const uint PA_READ_ESPI_ERR_BAD_CMD_CRC = 0x00000100;
public const uint PA_READ_ESPI_ERR_BAD_RSP_CRC = 0x00000200;
/* eSPI specific event */
public const uint PA_EVENT_ESPI_ALERT_RISING = 0x00010000;
public const uint PA_EVENT_ESPI_ALERT_FALLING = 0x00020000;
public const uint PA_EVENT_ESPI_RESET_RISING = 0x00040000;
public const uint PA_EVENT_ESPI_RESET_FALLING = 0x00080000;
public const uint PA_EVENT_ESPI_INBAND_RESET = 0x00100000;
public const uint PA_EVENT_ESPI_PACKET = 0x00200000;
/* eSPI transaction base information */
// enum PromiraEspiChannel  (from declaration above)
//     PA_ESPI_CHANNEL_UNKNOWN = -1
//     PA_ESPI_CHANNEL_PERIF   =  0
//     PA_ESPI_CHANNEL_VW      =  1
//     PA_ESPI_CHANNEL_OOB     =  2
//     PA_ESPI_CHANNEL_FLASH   =  3
//     PA_ESPI_CHANNEL_INDEP   =  4

// enum PromiraEspiEnumFreq  (from declaration above)
//     PA_ESPI_ENUM_FREQ_UNKNOWN = -1
//     PA_ESPI_ENUM_FREQ_20MHZ   =  0
//     PA_ESPI_ENUM_FREQ_25MHZ   =  1
//     PA_ESPI_ENUM_FREQ_33MHZ   =  2
//     PA_ESPI_ENUM_FREQ_50MHZ   =  3
//     PA_ESPI_ENUM_FREQ_66MHZ   =  4

/* eSPI packet information */
[StructLayout(LayoutKind.Sequential)]
public struct PromiraEspiPacketInfo {
    public PromiraEspiChannel  channel;
    public PromiraEspiEnumFreq enum_freq;
    public PromiraSpiIOMode    io_mode;
    public ushort              length;
    public ushort              cmd_length;
}

/* Read eSPI transaction */
public static int pa_espi_read (
    int                        conn,
    ref PromiraReadInfo        info,
    ref PromiraEspiPacketInfo  pkt_info,
    uint                       max_bytes,
    byte[]                     packet
)
{
    if (!PA_APP_LIBRARY_LOADED) return (int)PromiraAppStatus.PA_APP_INCOMPATIBLE_LIBRARY;
    uint packet_max_bytes = (uint)tp_min(max_bytes, packet.Length);
    return net_pa_espi_read(conn, ref info, ref pkt_info, packet_max_bytes, packet);
}

[StructLayout(LayoutKind.Sequential)]
public struct PromiraEspiStats {
    public uint ch_perif;
    public uint ch_vw;
    public uint ch_oob;
    public uint ch_flash;
    public uint get_cfg;
    public uint set_cfg;
    public uint get_sts;
    public uint pltf_reset;
    public uint alert;
    public uint reset;
    public uint cmd_crc;
    public uint resp_crc;
    public uint fltr_out_pkts;
    public uint fltr_out_cmds;
}

/* Get the statistics */
public static int pa_espi_stats_read (
    int                   conn,
    byte                  slave_id,
    ref PromiraEspiStats  stats
)
{
    if (!PA_APP_LIBRARY_LOADED) return (int)PromiraAppStatus.PA_APP_INCOMPATIBLE_LIBRARY;
    return net_pa_espi_stats_read(conn, slave_id, ref stats);
}

/* For channel match bitmask */
public const uint PA_ESPI_CHANNEL_MATCH_PERIF = 0x00000001;
public const uint PA_ESPI_CHANNEL_MATCH_VW = 0x00000002;
public const uint PA_ESPI_CHANNEL_MATCH_OOB = 0x00000004;
public const uint PA_ESPI_CHANNEL_MATCH_FLASH = 0x00000008;
public const uint PA_ESPI_CHANNEL_MATCH_INDEP = 0x00000010;
/* eSPI packet matching configuration */
[StructLayout(LayoutKind.Sequential)]
public struct PromiraEspiPacketMatch {
    public byte             ch_match_bitmask;
    public PromiraMatchType cmd_match_type;
    public byte             cmd_match_val;
}

/* Hardware filtering configuration */
public static int pa_espi_hw_filter_config (
    int                     conn,
    byte                    slave_id,
    PromiraEspiPacketMatch  pkt_match
)
{
    if (!PA_APP_LIBRARY_LOADED) return (int)PromiraAppStatus.PA_APP_INCOMPATIBLE_LIBRARY;
    return net_pa_espi_hw_filter_config(conn, slave_id, ref pkt_match);
}

/* Condtions for trigger */
public const uint PA_ESPI_ACT_ON_DIG_IN_MASK = 0x00000fff;
public const uint PA_ESPI_ACT_ON_ALERT = 0x00010000;
public const uint PA_ESPI_ACT_ON_RESET = 0x00040000;
public const uint PA_ESPI_ACT_ON_SLAVE_BITMASK = 0xf0000000;
/* Actions when triggered */
public const uint PA_ESPI_ACTION_DIG_OUT_MASK = 0x00000fff;
public const uint PA_ESPI_ACTION_TRIGGER = 0x80000000;
/* Simple trigger configuration */
public static int pa_espi_simple_trigger_config (
    int                     conn,
    uint                    act_on,
    PromiraEspiPacketMatch  pkt_match,
    uint                    actions
)
{
    if (!PA_APP_LIBRARY_LOADED) return (int)PromiraAppStatus.PA_APP_INCOMPATIBLE_LIBRARY;
    return net_pa_espi_simple_trigger_config(conn, act_on, ref pkt_match, actions);
}

/* Array_u08: 16 bytes long */
[StructLayout(LayoutKind.Sequential)]
public struct PromiraEspiAdvancedTrigBytes16 {
    public byte byte0;
    public byte byte1;
    public byte byte2;
    public byte byte3;
    public byte byte4;
    public byte byte5;
    public byte byte6;
    public byte byte7;
    public byte byte8;
    public byte byte9;
    public byte byteA;
    public byte byteB;
    public byte byteC;
    public byte byteD;
    public byte byteE;
    public byte byteF;
}

/* Array_u08: 2 bytes long */
[StructLayout(LayoutKind.Sequential)]
public struct PromiraEspiAdvancedTrigBytes2 {
    public byte byte0;
    public byte byte1;
}

/* Advanced Trigger Option 1 */
[StructLayout(LayoutKind.Sequential)]
public struct PromiraEspiAdvancedTrig1 {
    public byte                           cmd_byte;
    public byte                           cmd_cyc;
    public byte                           cmd_tag;
    public ushort                         cmd_len;
    public ulong                          cmd_addr;
    public PromiraEspiAdvancedTrigBytes16 cmd_data;
    public PromiraEspiAdvancedTrigBytes16 cmd_data_mask;
    public byte                           rsp_byte;
    public byte                           rsp_cyc;
    public byte                           rsp_tag;
    public ushort                         rsp_len;
    public ulong                          rsp_addr;
    public PromiraEspiAdvancedTrigBytes16 rsp_data;
    public PromiraEspiAdvancedTrigBytes16 rsp_data_mask;
    public PromiraEspiAdvancedTrigBytes2  sts_byte;
    public PromiraEspiAdvancedTrigBytes2  sts_mask;
    public byte                           trg_pin;
    public byte                           trg_pin_polarity;
    public byte                           trg_pin_direction;
    public byte                           cmd_byte_enable;
    public byte                           cmd_cyc_enable;
    public byte                           cmd_tag_enable;
    public byte                           cmd_len_enable;
    public byte                           cmd_addr_enable;
    public byte                           cmd_data_enable;
    public byte                           rsp_byte_enable;
    public byte                           rsp_cyc_enable;
    public byte                           rsp_tag_enable;
    public byte                           rsp_len_enable;
    public byte                           rsp_addr_enable;
    public byte                           rsp_data_enable;
    public byte                           sts_byte_enable;
    public byte                           trg_pin_enable;
    public byte                           lvl_select_enable;
    public byte                           lvl_select_immediate;
}

/* Advanced Trigger Option 2 */
[StructLayout(LayoutKind.Sequential)]
public struct PromiraEspiAdvancedTrig2 {
    public byte                           cmd_byte2;
    public byte                           cmd_cyc2;
    public byte                           cmd_tag2;
    public ushort                         cmd_len2;
    public ulong                          cmd_addr2;
    public PromiraEspiAdvancedTrigBytes16 cmd_data2;
    public PromiraEspiAdvancedTrigBytes16 cmd_data_mask2;
    public byte                           trg_pin_req;
    public byte                           trg_pin_req_polarity;
    public byte                           trg_pin_req_direction;
    public byte                           trg_pin_cmpl0;
    public byte                           trg_pin_cmpl0_polarity;
    public byte                           trg_pin_cmpl0_direction;
    public byte                           trg_pin_cmpl1;
    public byte                           trg_pin_cmpl1_polarity;
    public byte                           trg_pin_cmpl1_direction;
    public byte                           trg_pin_cmpl2;
    public byte                           trg_pin_cmpl2_polarity;
    public byte                           trg_pin_cmpl2_direction;
    public byte                           cmd_byte2_enable;
    public byte                           cmd_cyc2_enable;
    public byte                           cmd_tag2_enable;
    public byte                           cmd_len2_enable;
    public byte                           cmd_addr2_enable;
    public byte                           cmd_data2_enable;
    public byte                           succ_cmpl_enable;
    public byte                           unsucc_cmpl_enable;
    public byte                           trg_pin_req_enable;
    public byte                           trg_pin_cmpl0_enable;
    public byte                           trg_pin_cmpl1_enable;
    public byte                           trg_pin_cmpl2_enable;
}

/* Advanced Trigger Option Error */
[StructLayout(LayoutKind.Sequential)]
public struct PromiraEspiAdvancedTrigError {
    public byte err_code;
    public byte err_code_enable;
}

/* Advanced Trigger Configuration */
public static int pa_espi_adv_trig_config (
    int                           conn,
    byte                          slave_id,
    PromiraEspiAdvancedTrig1      pkt_adv_trig1_level1,
    PromiraEspiAdvancedTrig1      pkt_adv_trig1_level2,
    PromiraEspiAdvancedTrig1      pkt_adv_trig1_level3,
    PromiraEspiAdvancedTrig1      pkt_adv_trig1_level4,
    PromiraEspiAdvancedTrig2      pkt_adv_trig2,
    PromiraEspiAdvancedTrigError  pkt_adv_trig_error
)
{
    if (!PA_APP_LIBRARY_LOADED) return (int)PromiraAppStatus.PA_APP_INCOMPATIBLE_LIBRARY;
    return net_pa_espi_adv_trig_config(conn, slave_id, ref pkt_adv_trig1_level1, ref pkt_adv_trig1_level2, ref pkt_adv_trig1_level3, ref pkt_adv_trig1_level4, ref pkt_adv_trig2, ref pkt_adv_trig_error);
}


/*=========================================================================
| NATIVE DLL BINDINGS
 ========================================================================*/
[DllImport ("promana")]
private static extern int net_pa_app_connect (string net_addr);

[DllImport ("promana")]
private static extern int net_pa_app_disconnect (int conn);

[DllImport ("promana")]
private static extern int net_pa_app_version (int conn, ref PromiraAppVersion version);

[DllImport ("promana")]
private static extern IntPtr net_pa_app_status_string (int status);

[DllImport ("promana")]
private static extern int net_pa_app_features (int conn);

[DllImport ("promana")]
private static extern int net_pa_phy_target_power (int conn, byte config);

[DllImport ("promana")]
private static extern float net_pa_phy_level_shift (int conn, float level);

[DllImport ("promana")]
private static extern int net_pa_capture_start (int conn, PromiraProtocol protocol, PromiraTriggerMode trig_mode);

[DllImport ("promana")]
private static extern int net_pa_capture_stop (int conn);

[DllImport ("promana")]
private static extern int net_pa_capture_trigger (int conn);

[DllImport ("promana")]
private static extern int net_pa_capture_trigger_wait (int conn, int timeout_ms, ref PromiraCaptureStatus status);

[DllImport ("promana")]
private static extern int net_pa_capture_status (int conn, ref PromiraCaptureStatus status, ref PromiraCaptureBufferStatus buf_status);

[DllImport ("promana")]
private static extern int net_pa_capture_buffer_config (int conn, uint pretrig_kb, uint capture_kb);

[DllImport ("promana")]
private static extern int net_pa_capture_buffer_config_query (int conn, ref uint pretrig_kb, ref uint capture_kb);

[DllImport ("promana")]
private static extern int net_pa_digital_io_config (int conn, uint enable, uint direction, uint polarity);

[DllImport ("promana")]
private static extern int net_pa_digital_io_config_query (int conn, ref uint enable, ref uint direction, ref uint polarity);

[DllImport ("promana")]
private static extern int net_pa_espi_operating_config (int conn, byte slave_id, ref PromiraEspiOperatingCfg cfg);

[DllImport ("promana")]
private static extern int net_pa_espi_operating_config_read (int conn, byte slave_id, ref PromiraEspiOperatingCfg cfg);

[DllImport ("promana")]
private static extern int net_pa_espi_read (int conn, ref PromiraReadInfo info, ref PromiraEspiPacketInfo pkt_info, uint max_bytes, [Out] byte[] packet);

[DllImport ("promana")]
private static extern int net_pa_espi_stats_read (int conn, byte slave_id, ref PromiraEspiStats stats);

[DllImport ("promana")]
private static extern int net_pa_espi_hw_filter_config (int conn, byte slave_id, ref PromiraEspiPacketMatch pkt_match);

[DllImport ("promana")]
private static extern int net_pa_espi_simple_trigger_config (int conn, uint act_on, ref PromiraEspiPacketMatch pkt_match, uint actions);

[DllImport ("promana")]
private static extern int net_pa_espi_adv_trig_config (int conn, byte slave_id, ref PromiraEspiAdvancedTrig1 pkt_adv_trig1_level1, ref PromiraEspiAdvancedTrig1 pkt_adv_trig1_level2, ref PromiraEspiAdvancedTrig1 pkt_adv_trig1_level3, ref PromiraEspiAdvancedTrig1 pkt_adv_trig1_level4, ref PromiraEspiAdvancedTrig2 pkt_adv_trig2, ref PromiraEspiAdvancedTrigError pkt_adv_trig_error);


} // class PromanaApi

} // namespace TotalPhase
